@@ -240,9 +240,11 @@ gok_input_open (GokInput *input)
   if (window && window->window && 
       input && input->info && (input->info->use == IsXExtensionDevice))
     {
+      gdk_error_trap_push ();
       input->device = XOpenDevice(GDK_WINDOW_XDISPLAY (window->window), 
 				  input->info->id);
-      input->open = TRUE;
+      if (!gdk_error_trap_pop ())
+	  input->open = TRUE;
     }
 #endif
   if (!input && !input->open)
