@@ -104,16 +104,6 @@ gboolean gok_page_actions_initialize (GladeXML* xml)
 	
 		gtk_combo_set_popdown_strings (GTK_COMBO (pComboBox), items);
 		g_list_free (items);
-
-		slist = pDevice;
-		while ((slist != NULL) && (slist->data != NULL))
-		{
-			GokInput *input;
-			input = (GokInput *) slist->data;
-			gok_input_free (input); 
-			slist = slist->next;
-		}
-		g_slist_free (pDevice);
 	}
 
 	gok_page_actions_enable_radios_type (FALSE);
@@ -347,8 +337,7 @@ void gok_page_actions_toggle_type_valuator (gboolean Pressed)
 
 	if (Pressed == TRUE)
 	{
-	  /* update the action from the controls */
-	  m_pActionCurrent->Type = ACTION_TYPE_DWELL;
+		/* TODO: consider disabling some controls here */
 	}	
 	pSpinButton = (GtkSpinButton*) glade_xml_get_widget (gok_settingsdialog_get_xml(), 
 			"pointer_delay_spinbutton");
@@ -459,8 +448,7 @@ void gok_page_actions_button_clicked_change_name ()
 				{
 					/* yes the name was changed */
 					g_free (m_pActionCurrent->pDisplayName);
-					m_pActionCurrent->pDisplayName = (gchar*)g_malloc (strlen (pNewName) + 1);
-					strcpy (m_pActionCurrent->pDisplayName, pNewName);
+					m_pActionCurrent->pDisplayName = g_strdup (pNewName);
 		
 					/* add the new name to the combo box list */
 					gok_page_actions_fill_combo_action_names();
@@ -486,8 +474,8 @@ void gok_page_actions_button_clicked_new ()
 {
 	GokAction* pActionNew;
 	int count;
-	gchar bufferName[200];
-	gchar bufferDisplayName[200];
+	gchar *bufferName;
+	gchar *bufferDisplayName;
 	
 	g_assert (m_pActionCurrent != NULL);
 	
@@ -495,20 +483,20 @@ void gok_page_actions_button_clicked_new ()
 	/* it will be something like "new action 1" */
 	for (count = 1; count < 100; count++)
 	{
-		sprintf (bufferDisplayName, _("New Action %d"), count);
-		sprintf (bufferName, "new_action_%d", count);
+		bufferDisplayName = g_strdup_printf (_("New Action %d"), count);
+		bufferName = g_strdup_printf ("new_action_%d", count);
 		if ((gok_action_find_action (bufferDisplayName, TRUE) == NULL) &&
 			(gok_action_find_action (bufferName, FALSE) == NULL))
 		{
 			break;
 		}
+		g_free (bufferDisplayName);
+		g_free (bufferName);
 	}
 	
 	pActionNew = gok_action_new();
-	pActionNew->pName = (gchar*)g_malloc (strlen (bufferName) + 1);
-	strcpy (pActionNew->pName, bufferName);
-	pActionNew->pDisplayName = (gchar*)g_malloc (strlen (bufferDisplayName) + 1);
-	strcpy (pActionNew->pDisplayName, bufferDisplayName);
+	pActionNew->pName = bufferName;
+	pActionNew->pDisplayName = bufferDisplayName;
 	
 	/* make the new action the same as the current action */
 	if (m_pActionCurrent != NULL)
@@ -727,7 +715,7 @@ void gok_page_actions_input_device_changed (GtkEditable* pEditControl)
 
 	/* get the name of the input device from the combo box */
 	pStrComboInputDeviceName = gtk_editable_get_chars (pEditControl, 0, -1);
-	if (strlen (pStrComboInputDeviceName) == 0) {
+	if (pStrComboInputDeviceName && strlen (pStrComboInputDeviceName) == 0) {
 		g_free (pStrComboInputDeviceName);
 	}
 	else
@@ -743,7 +731,6 @@ void gok_page_actions_input_device_changed (GtkEditable* pEditControl)
 			gok_page_actions_set_changed ( TRUE );
 			gok_input_set_extension_device_by_name (
 				pStrComboInputDeviceName);
-			gok_input_free (pInput);
 		}
 		g_free (pStrComboInputDeviceName);
 	}
