@@ -123,7 +123,7 @@ static gint m_GeometryY = -1;
 /* pointer to the gok command predictor */
 static Gok_Predictor m_pCommandPredictor = NULL;
 
-/* poptOption storage structure */
+/* command line aguments storage structure */
 typedef struct _GokArgs {
 	char* accessmethodname;
 	char* inputdevicename;
@@ -170,83 +170,83 @@ void       gok_input_restore_corepointer (void);
 static gint _screen_height, _screen_width;
 
 
-static struct poptOption options[] = {
+static const GOptionEntry options[] = {
 	{
 		"access-method", 
 		'a',
-		POPT_ARG_STRING, 
-		&gok_args.accessmethodname, 
 		0,
+		G_OPTION_ARG_STRING, 
+		&gok_args.accessmethodname, 
 		N_("Use the specified access method. NAME is a string and can be found in the various access method files (.xam) assigned to the \"name\" property of <gok:accessmethod> tag. Note this is not necessarily the same as the name of the .xam file. (See --list-accessmethods)"), 
 		N_("NAME")
 	},
 	{
 		"editor", 
-		'e', 
-		POPT_ARG_NONE, 
+		'e',
+		0,	
+		G_OPTION_ARG_NONE, 
 		&gok_args.display_keyboard_editor, 
-		0,
 		N_("Start the GOK keyboard editor"), 
 		NULL},
 	{
 		"extras", 
 		'\0', 
-		POPT_ARG_NONE, 
-		&gok_args.use_extras, 
 		0,
+		G_OPTION_ARG_NONE, 
+		&gok_args.use_extras, 
 		N_("Use special, but possibly unstable, gok stuff"),
 		NULL},
 	{
 		"geometry", 
 		'\0', 
-		POPT_ARG_STRING, 
-		&gok_args.geometry, 
 		0,
+		G_OPTION_ARG_STRING, 
+		&gok_args.geometry, 
 		N_("Whenever --geometry is not used gok remembers its position between invocations and starts in the position that it had when it was last shutdown.  When --geometry is used gok positions itself within the rectangular area of screen described by the given X11 geometry specification.  When --geometry is used gok does not remember its position when it shuts down.  This behaviour can be changed with the --remembergeometry flag which forces gok to remember its position when shutdown even when it was started with --geometry."),
 		N_("GEOMETRY")
 	},
 	{
 		"input-device", 
 		'i', 
-		POPT_ARG_STRING, 
-		&gok_args.inputdevicename, 
 		0,
+		G_OPTION_ARG_STRING, 
+		&gok_args.inputdevicename, 
 		N_("Use the specified input device"), 
 		N_("DEVICENAME")
 	},
 	{
 		"keyboard", 
 		'k', 
-		POPT_ARG_STRING, 
-		&gok_args.mainkeyboardname, 
 		0,
+		G_OPTION_ARG_STRING, 
+		&gok_args.mainkeyboardname, 
 		N_("Start GOK with the specified keyboard."), 
 		N_("KEYBOARDNAME")
 	},
 	{
 		"list-accessmethods", 
 		'\0', 
-		POPT_ARG_NONE, 
-		&gok_args.list_accessmethods, 
 		0,
+		G_OPTION_ARG_NONE, 
+		&gok_args.list_accessmethods, 
 		N_("List the access methods that can be used as options to other arguments."),
 		NULL
 	},
 	{
 		"list-actions", 
 		'\0', 
-		POPT_ARG_NONE, 
-		&gok_args.list_actions, 
 		0,
+		G_OPTION_ARG_NONE, 
+		&gok_args.list_actions, 
 		N_("List the actions that can be used as options to other arguments."),
 		NULL
 	},
 	{
 		"login", 
 		'l', 
-		POPT_ARG_NONE, 
-		&gok_args.is_login, 
 		0,
+		G_OPTION_ARG_NONE, 
+		&gok_args.is_login, 
 		N_("GOK will be used to login"),
 		NULL
 	},
@@ -254,9 +254,9 @@ static struct poptOption options[] = {
 	{
 		"debugnameless", 
 		'\0', 
-		POPT_ARG_NONE, 
-		&gok_args.debug_nameless, 
 		0,
+		G_OPTION_ARG_NONE, 
+		&gok_args.debug_nameless, 
 		N_("GOK will display keys for nameless GUI. Useful for debugging."),
 		NULL
 	},
@@ -264,9 +264,9 @@ static struct poptOption options[] = {
 	{
 		"remembergeometry", 
 		'\0', 
-		POPT_ARG_NONE,
-		&gok_args.remember_geometry, 
 		0,
+		G_OPTION_ARG_NONE,
+		&gok_args.remember_geometry, 
 		N_("Can be used with --geometry.  Forces GOK to remember its position when shutdown even when it was started with --geometry.  Please see the discussion under the --geometry flag for more information."),
 		NULL
 	},
@@ -275,9 +275,9 @@ static struct poptOption options[] = {
 		/* note: not enough info for 5-switch directed scanning */
 		"scan-action", 
 		'\0', 
-		POPT_ARG_STRING, 
-		&gok_args.scanactionname, 
 		0,
+		G_OPTION_ARG_STRING, 
+		&gok_args.scanactionname, 
 		N_("Start GOK and hook this action to scan operations. (See --list-actions)"), 
 		NULL
 	},
@@ -285,33 +285,33 @@ static struct poptOption options[] = {
 		/* primarily for use at login since there is no gconf */
 		"select-action", 
 		'\0', 
-		POPT_ARG_STRING, 
-		&gok_args.selectactionname, 
 		0,
+		G_OPTION_ARG_STRING, 
+		&gok_args.selectactionname, 
 		N_("Start GOK and hook this action to select operations. (See --list-actions)"), 
 		NULL
 	},
 	{
 		"settings", 
 		's', 
-		POPT_ARG_NONE, 
-		&gok_args.display_settings_dialog, 
 		0,
+		G_OPTION_ARG_NONE, 
+		&gok_args.display_settings_dialog, 
 		N_("Open the settings dialog box when GOK starts"),
 		NULL
 	},
 	{
 		"valuator-sensitivity", 
 		'\0', 
-		POPT_ARG_DOUBLE, 
-		&gok_args.valuator_sensitivity, 
 		0,
+		G_OPTION_ARG_DOUBLE, 
+		&gok_args.valuator_sensitivity, 
 		N_("A multiplier to be applied to input device valuator events before processing"),
 		NULL
 	},
 
 	/* End the list */
-	{NULL, '\0', 0, NULL, 0, NULL, NULL}
+	{	NULL}
 };
 
 static int segfaults = 0;
@@ -348,7 +348,7 @@ static int gok_sig_handler(int sig)
 static void
 gok_args_init (GokArgs *args)
 {
-	/* initialize command line option (popt) storage */
+	/* initialize command line option storage */
 	args->accessmethodname = NULL;
 	args->inputdevicename = NULL;
 	args->mainkeyboardname = NULL;
@@ -536,6 +536,7 @@ gint
 gok_main_open(gint argc, gchar *argv[])
 {
 	GnomeProgram *gok_program;
+	GOptionContext *context;
 
 	/* initialize member data */
 	m_pForegroundWindowAccessible = NULL;
@@ -546,10 +547,13 @@ gok_main_open(gint argc, gchar *argv[])
 
 	gok_args_init (&gok_args);
 
+	context	= g_option_context_new (_("The GNOME On-screen Keyboard"));
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	/* Initialize Gnome */
 	gok_program = gnome_program_init (PACKAGE, VERSION,
 		LIBGNOMEUI_MODULE, argc, argv,
-		GNOME_PARAM_POPT_TABLE, options,
+		GNOME_PARAM_GOPTION_CONTEXT, context,
 		GNOME_PROGRAM_STANDARD_PROPERTIES,
 	        GNOME_PARAM_APP_DATADIR, DATADIR"/gok", 
 		LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
@@ -805,14 +809,10 @@ gok_main_open(gint argc, gchar *argv[])
 	   we use the login keyboard, otherwise the regular main keyboard */
 	if (gok_args.mainkeyboardname == NULL) {
 		if (gok_args.is_login) {
-			gok_args.mainkeyboardname = (gchar*) 
-				g_malloc (strlen (GOKLOGINKEYBOARDNAME) + 1);
-			strcpy (gok_args.mainkeyboardname, (gchar *) GOKLOGINKEYBOARDNAME);
+			gok_args.mainkeyboardname = (gchar*)  g_strdup ((gchar *) GOKLOGINKEYBOARDNAME);
 		}
 		else {
-			gok_args.mainkeyboardname = (gchar*) 
-				g_malloc (strlen (GOKMAINKEYBOARDNAME) + 1);
-			strcpy (gok_args.mainkeyboardname, (gchar *) GOKMAINKEYBOARDNAME);
+			gok_args.mainkeyboardname = (gchar*) g_strdup((gchar *) GOKMAINKEYBOARDNAME);
 		}
 	}
 
@@ -846,6 +846,8 @@ gok_log ("finished check");
 	signal(SIGTERM, (void (*)(int))gok_sig_handler);
 	signal(SIGINT,  (void (*)(int))gok_sig_handler);
 
+	g_object_unref (gok_program);
+
 	return 0;
 }
 
@@ -1980,7 +1982,7 @@ gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 	gchar* filename;
 	gchar* complete_path;
 	GList* langlist = NULL;
-	gchar dirtemp[1024]; /* no reliable way to avoid magic # here */
+	gchar *dirtemp;
 
 	gok_log_enter();
 
@@ -1991,22 +1993,20 @@ gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 	if (langlist != NULL) {
 		gint i = 0;
 		guint l = g_list_length(langlist);
-		while ((pDirectoryKeyboards == NULL) && (i < l)) {
-			strncpy(dirtemp, directory, 950);
-			strncat(dirtemp, G_DIR_SEPARATOR_S, 1);
-			strncat(dirtemp, g_list_nth_data(langlist, i),70);
+		while ((i < l)) {
+			dirtemp = g_build_filename (directory, g_list_nth_data(langlist, i), NULL);
 			pDirectoryKeyboards = opendir (dirtemp);
+			if (pDirectoryKeyboards) 
+				break;
+			g_free (dirtemp);
 			i++;
 		}
 	}
 
 	if (pDirectoryKeyboards == NULL) { /* no locale specific directory found */
 		pDirectoryKeyboards = opendir (directory); /* use default */
+		dirtemp = g_strdup (directory);
 	}
-	else {
-		directory = dirtemp; /* used later */
-	}
-		
 
 	/* get a list of all the files from the given directory name */
 	if (pDirectoryKeyboards == NULL)
@@ -2031,7 +2031,7 @@ gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 		     && (strcmp(filename + (strlen(filename)-4), ".kbd") == 0))
 		{
 			/* read the keyboard file and add to list */
-                        complete_path = g_build_filename (directory, filename,
+                        complete_path = g_build_filename (dirtemp, filename,
                                                           NULL);
 			gok_log ("complete_path = %s", complete_path);
 			keyboard = gok_append_keyboard_from_file (keyboard, complete_path);
@@ -2042,6 +2042,7 @@ gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 		}
 	}
  	g_free (pDirectoryKeyboards);
+	g_free (dirtemp);
 	gok_log_leave();
 	return pKeyboardFirst;
 }
@@ -2627,10 +2628,9 @@ gok_main_close()
 void gok_main_display_error (gchar* ErrorString)
 {
 	GtkWidget* pDialog;
-	gchar buffer [1024];
+	gchar* buffer;
 	
-	strcpy (buffer, _("Sorry, GOK can't run because:\n"));
-	strcat (buffer, ErrorString);
+	buffer = g_strdup_printf (_("Sorry, GOK can't run because:\n%s"), ErrorString);
 	
 	pDialog = gtk_message_dialog_new (NULL,
 		GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -2641,6 +2641,7 @@ void gok_main_display_error (gchar* ErrorString)
 	gtk_window_set_title (GTK_WINDOW (pDialog), _("GOK Fatal Error"));
 	gtk_dialog_run (GTK_DIALOG (pDialog));
 	gtk_widget_destroy (pDialog);
+	g_free (buffer);
 }
 
 /**
