@@ -26,6 +26,7 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#include <glib-object.h>
 #include <cspi/spi.h>
 #include <libspi/Accessibility.h>
 #include <libspi/accessible.h>
@@ -88,20 +89,24 @@ gboolean gok_main_get_extras (void);
 gboolean gok_main_get_login (void);
 gchar* gok_main_get_inputdevice_name (void);
 void gok_main_check_accessibility (void);
-void gok_main_warn_if_corepointer_mode (gchar *message, gboolean always);
+void gok_main_warn_corepointer (gboolean always, gboolean configuring, gboolean opt_out);
+gboolean gok_main_warn (gchar *message, gboolean always, GCallback cancelfunc, GCallback okfunc, gboolean opt_out);
 gboolean gok_main_safe_mode (void);
+void gok_main_close_warning (void);
+void gok_main_center_corepointer (GtkWidget *widget);
 
 /* private */
 void gok_main_app_change_listener (Accessible* pAccessible);
 void gok_main_window_change_listener (Accessible* pAccessible);
 void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long timestamp);
 void gok_main_button_listener (gint button, gint state, long mods, long timestamp);
-void gok_main_mousebutton_listener (gint button, gint state, long mods, long timestamp);
+void gok_main_mouse_button_listener (gint button, gint state, long mods, long timestamp);
 void gok_main_close (void);
 gboolean gok_main_get_use_geometry (void);
 void gok_main_get_geometry (GdkRectangle* pRectangle);
 void gok_main_update_struts (gint width, gint height, gint x, gint y);
 void gok_main_set_wm_dock (gboolean is_dock);
+gboolean gok_main_get_debug_nameless (void);
 
 #ifdef __cplusplus
 }
