@@ -35,8 +35,8 @@
 
 /* private functions */
 static gchar    WordCompleteSaveDictionary     (void);
-static gint     WordCompleteSaveLoop           (Node* pDictLetter, gchar* pWord, FILE* pFile);
-static void     WordCompletePredictLoop        (const gchar* pWord, Node* pNode, gchar* pPrediction);
+static gint     WordCompleteSaveLoop           (Node* pDictLetter, GString* pWord, FILE* pFile);
+static void     WordCompletePredictLoop        (const gchar* pWord, Node* pNode, GString *pPrediction);
 static void     WordCompleteDeleteChildren     (Node* pNode);
 static gboolean WordCompleteAddWord            (GokWordComplete *complete, 
 						const gchar* pWord, gint priority, gint state);
@@ -53,7 +53,7 @@ static gchar**  WordCompletePredictString      (GokWordComplete *complete, const
 /* a prediction */
 struct PredictNode
 {
-	gchar word[MAXWORDLENGTH];
+	gchar* word;
 	gint priority;
 };
 
@@ -67,9 +67,6 @@ typedef enum
 	WORD_CASE_TITLE
 } WordPredictionCaseType;
 
-/* this will hold the string of predictions that is returned to the caller */
-static gchar m_buffer [MAXWORDLENGTH * MAXPREDICTIONS + MAXPREDICTIONS + 1];
-
 /* will be TRUE if "WordCompleteOpen" has been called */
 static gint bWordCompleteOpened = FALSE; 
 
@@ -500,14 +497,11 @@ static gchar**
 WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint numberPredictions)
 {
 	gchar **word_predict_list = NULL;
+	GString *buffer;
 
 	/* make the prediction */
-
-	gchar bufferLowercase[MAXWORDLENGTH + 1];
-	gint lengthWord;
-	gint count, i, j;
+	gint i, j;
 	gchar* pWordLowercase;
-	gchar buffer [MAXWORDLENGTH + 1];
 
 	gint bMadePrediction;
 	gint highPriority;
@@ -537,25 +531,18 @@ WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint n
 	  m_Predictions[i].priority = 0;
 	}
 
-	/* convert the given string to lowercase (cuz our dictionary is all lower case) */
-	/* FIXME: not UTF-8 ready! */
-	lengthWord = strlen (pWord);
-	for (count = 0; count < lengthWord; count++)
-	{
-	  bufferLowercase[count] = tolower (pWord[count]);
-	}
-	bufferLowercase[count] = 0;
-	pWordLowercase = &bufferLowercase[0];
+	pWordLowercase = g_utf8_strdown (pWord, -1);
 
 	/* search the trie for the given word */
 	/* find the first node with the word's first letter */
 	pNode = m_FirstNode.pFirstChildNode;
+	buffer = g_string_new ("");
 	while (pNode != NULL)
 	{
 		if (pNode->letter == pWordLowercase[0])
 		{
-			buffer[0] = pWordLowercase[0];
-			buffer[1] = 0;
+			g_string_assign (buffer, "");
+			g_string_append_c (buffer, pWordLowercase[0]);
 			pWordLowercase++;
 			/* create the predictions */
 			WordCompletePredictLoop (pWordLowercase, pNode, buffer);
@@ -563,6 +550,7 @@ WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint n
 		}
 		pNode = pNode->pNextNode;
 	}
+	g_string_free (buffer, TRUE);
 
 	/* look through our array of predictions and find the highest priority ones */
 	bMadePrediction = FALSE;
@@ -592,6 +580,8 @@ WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint n
 
 	word_predict_list[i] = NULL;
 
+	g_free (pWordLowercase);
+
 	/* were there any predictions? */
 	if (bMadePrediction == FALSE)
 	{
@@ -601,7 +591,7 @@ WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint n
 
 	/* post-process the predictions to match the case of the input */
 	WordCompleteApplyCase (pWord, word_predict_list, numberPredictions);
-
+	    
 	return word_predict_list; /* predictions made! */
 }
 
@@ -613,11 +603,11 @@ WordCompletePredictString (GokWordComplete *complete, const gchar* pWord, gint n
  * Each word is stored in an array by calling "AddPrediction".
  */
 static void 
-WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPrediction)
+WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, GString* pPrediction)
 {
 	if (pNode != NULL)
 	{
-		int length = (int)strlen (pPrediction);
+		int length = (int)strlen (pPrediction->str);
 
 		/* are we at the end of the given word? */
 		if (pWordGiven[0] == 0)
@@ -627,12 +617,11 @@ WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPredictio
 			Node* pChildNode = pNode->pFirstChildNode;
 			while (pChildNode != NULL)
 			{
-				pPrediction[length] = pChildNode->letter;
-				pPrediction[length + 1] = 0;
-                           /* priority > 0 means that this is the last letter in a word */
+				g_string_append_c (pPrediction, pChildNode->letter);
+                               /* priority > 0 means that this is the last letter in a word */
 				if (pChildNode->priority != 0)
 				{
-					WordCompleteAddPrediction (pPrediction, pChildNode->priority);
+					WordCompleteAddPrediction (pPrediction->str, pChildNode->priority);
 				}
 				/* add all the children's children */
 				WordCompletePredictLoop (pWordGiven, pChildNode, pPrediction);
@@ -650,8 +639,7 @@ WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPredictio
 				if (pChildNode->letter == pWordGiven[0])
 				{
 					/* yes, move on to next letter in the given word */
-					pPrediction[length] = pWordGiven[0];
-					pPrediction[length + 1] = 0;
+					g_string_append_c (pPrediction, pWordGiven[0]);
 					pWordGiven++;
 
 					/* recurse */
@@ -662,7 +650,7 @@ WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPredictio
 			}
 		}
 
-		pPrediction[length] = 0;
+		g_string_set_size (pPrediction, length);
 	}
 }
 
@@ -676,9 +664,9 @@ WordCompletePredictLoop (const gchar* pWordGiven, Node* pNode, gchar* pPredictio
 static void 
 WordCompleteAddPrediction (gchar* pWord, gint priority)
 {
-    gint x;
-    gint lowPriorityIndex;
-    gint lowPriority;
+	gint x;
+	gint lowPriorityIndex;
+	gint lowPriority;
 
 	/* can the prediction get added to a blank spot in the array? */
 	for (x = 0; x < MAXPREDICTIONS; x++)
@@ -687,7 +675,8 @@ WordCompleteAddPrediction (gchar* pWord, gint priority)
 		if (m_Predictions[x].priority == 0)
 		{
 			m_Predictions[x].priority = priority;
-			strcpy (m_Predictions[x].word, pWord);
+			g_free (m_Predictions[x].word);
+			m_Predictions[x].word = g_strdup (pWord);
 			return;
 		}
 	}
@@ -710,7 +699,8 @@ WordCompleteAddPrediction (gchar* pWord, gint priority)
 	{
 		/* yes, replace the low priority prediction with this prediction*/
 		m_Predictions[lowPriorityIndex].priority = priority;
-		strcpy (m_Predictions[lowPriorityIndex].word, pWord);
+		g_free (m_Predictions[lowPriorityIndex].word);
+		m_Predictions[lowPriorityIndex].word = g_strdup (pWord);
 	}
 	/* no, the new prediction priority was less than all predictions in the array */
 	/* so it doesn't get added to the array */
@@ -777,7 +767,7 @@ WordCompleteSaveDictionary()
 {
 	FILE* pFile;
 	Node* pNode;
-	gchar buffer [MAXWORDLENGTH];
+	GString *buffer;
 
 	/* open the dictionary file */
 	pFile = fopen (dictionary_full_path, "wt");
@@ -792,13 +782,14 @@ WordCompleteSaveDictionary()
 	  fclose (pFile);
 	  return FALSE;
 	}
-
+	
 	fprintf (stderr, "saving dictionary...\n");
 	/* iterate through each child node */
 	pNode = m_FirstNode.pFirstChildNode;
+	buffer = g_string_new ("");
 	while (pNode != NULL)
-	{
-		buffer[0] = 0;
+	{	
+		g_string_assign (buffer, "");
 		if (WordCompleteSaveLoop (pNode, buffer, pFile) == FALSE)
 		{
 			fclose (pFile);
@@ -806,6 +797,7 @@ WordCompleteSaveDictionary()
 		}
 		pNode = pNode->pNextNode;
 	}
+	g_string_free (buffer, TRUE);
 
 	fclose (pFile);
 	return TRUE;
@@ -817,19 +809,18 @@ WordCompleteSaveDictionary()
  * Recursive loop for saving all dictionary words to disk.
  */
 static int 
-WordCompleteSaveLoop (Node* pNode, gchar* pBuffer, FILE* pFile)
+WordCompleteSaveLoop (Node* pNode, GString* pBuffer, FILE* pFile)
 {
-    gint lengthWord;
-    Node* pLetter;
+	gint lengthWord;
+	Node* pLetter;
 
-    lengthWord = strlen (pBuffer);
-    pBuffer[lengthWord] = pNode->letter;
-    pBuffer[lengthWord+1] = 0;
+	lengthWord = strlen (pBuffer->str);
+	g_string_append_c (pBuffer, pNode->letter);
 
 	if (pNode->priority != 0) /* priority > 0 means this is the end of a word */
 	{
 		/* write a word to the file */
-		if (fprintf (pFile, "%s\t%d\t%d\n", pBuffer, pNode->priority, pNode->state) < 0)
+		if (fprintf (pFile, "%s\t%d\t%d\n", pBuffer->str, pNode->priority, pNode->state) < 0)
 		{
 			return FALSE;
 		}
@@ -845,7 +836,7 @@ WordCompleteSaveLoop (Node* pNode, gchar* pBuffer, FILE* pFile)
 		pLetter = pLetter->pNextNode;
 	}
 
-	pBuffer[lengthWord] = 0;
+	g_string_set_size (pBuffer, lengthWord);
 
 	return TRUE;
 }
